@@ -8,11 +8,12 @@
 	(*pretty-print-spacing* 2)
 	(*pretty-print-float-format* "~,4F")
 	(*pretty-print-left-margin* 0)
-	(any? (lambda (f sequence) ; this and every? ought to be built-in!
-		(call-with-exit
-		 (lambda (return) 
-		   (for-each (lambda (arg) (if (f arg) (return #t))) sequence)
-		   #f)))))
+	(*pretty-print-cycles* #t))
+
+    (define (any? f sequence)
+      (and (pair? sequence)
+	   (or (f (car sequence))
+	       (any? f (cdr sequence)))))
 
     (define pretty-print-1
       (letrec ((messy-number (lambda (z)
@@ -434,7 +435,43 @@
 		     (if (rational? obj)
 			 (write obj port)
 			 (display (messy-number obj) port)))
+
+		    ((or (int-vector? obj)
+			 (float-vector? obj))
+		     (if (> (length (vector-dimensions obj)) 1)
+			 (write obj port)
+			 (let* ((objstr (object->string obj))
+				(strlen (length objstr)))
+			   (if (< (+ column strlen) *pretty-print-length*)
+			       (display objstr port)
+			       (let ((name-len (if (int-vector? obj) 10 12)))
+				 (display (if (int-vector? obj) "(int-vector " "(float-vector ") port)
+				 (set! column (+ column 2))
+				 (if (< (- *pretty-print-length* column) 30)
+				     (spaces port column)
+				     (set! column (+ column name-len)))
+				 (do ((len (min (length obj) (*s7* 'print-length)))
+				      (col column)
+				      (i 0 (+ i 1)))
+				     ((= i len)
+				      (if (> len (*s7* 'print-length))
+					  (write-string " ...)" port)
+					  (write-char #\) port)))
+				   (let* ((numstr (number->string (obj i)))
+					  (numlen (length numstr)))
+				     (if (not (= col column))
+					 (if (> (+ col numlen) *pretty-print-length*)
+					     (begin
+					       (spaces port column)
+					       (set! col column))
+					     (display #\space port)))
+				     (display numstr port)
+				     (set! col (+ col numlen 1)))))))))
 		    
+		    ((and *pretty-print-cycles*
+			  (pair? (cyclic-sequences obj)))
+		     (format port "~W" obj))
+
 		    ((hash-table? obj)
 		     (display "(hash-table" port)
 		     (for-each (lambda (field)
@@ -460,38 +497,6 @@
 				     obj)
 			   (write-char #\) port))))
 		    
-		    ((or (int-vector? obj)
-			 (float-vector? obj))
-		     (if (> (length (vector-dimensions obj)) 1)
-			 (write obj port)
-			 (let* ((objstr (object->string obj))
-				(strlen (length objstr)))
-			   (if (< (+ column strlen) *pretty-print-length*)
-			       (display objstr port)
-			       (let ((name-len (if (int-vector? obj) 10 12)))
-				 (display (if (int-vector? obj) "(int-vector " "(float-vector ") port)
-				 (set! column (+ column 2))
-				 (if (< (- *pretty-print-length* column) 30)
-				     (spaces port column)
-				     (set! column (+ column name-len)))
-				 (do ((len (min (length obj) (let-ref *s7* 'print-length)))
-				      (col column)
-				      (i 0 (+ i 1)))
-				     ((= i len)
-				      (if (> len (let-ref *s7* 'print-length))
-					  (write-string " ...)" port)
-					  (write-char #\) port)))
-				   (let* ((numstr (number->string (obj i)))
-					  (numlen (length numstr)))
-				     (if (not (= col column))
-					 (if (> (+ col numlen) *pretty-print-length*)
-					     (begin
-					       (spaces port column)
-					       (set! col column))
-					     (display #\space port)))
-				     (display numstr port)
-				     (set! col (+ col numlen 1)))))))))
-		    
 		    ((vector? obj)
 		     (if (> (length (vector-dimensions obj)) 1)
 			 (write obj port)
@@ -505,11 +510,11 @@
 				 (if (< (- *pretty-print-length* column) 30)
 				     (spaces port column)
 				     (set! column (+ column 6)))
-				 (do ((len (min (length obj) (let-ref *s7* 'print-length)))
+				 (do ((len (min (length obj) (*s7* 'print-length)))
 				      (col column)
 				      (i 0 (+ i 1)))
 				     ((= i len)
-				      (if (> len (let-ref *s7* 'print-length))
+				      (if (> len (*s7* 'print-length))
 					  (write-string " ...)" port)
 					  (write-char #\) port)))
 				   (let ((olen (length (object->string (obj i)))))
@@ -653,9 +658,9 @@
 (define (pp obj)
   (call-with-output-string
    (lambda (p)
-      (if (keyword? obj)
-	  (symbol->string obj)
-	  (pretty-print obj p)))))
+     (if (keyword? obj)
+	 (display obj p)
+	 (pretty-print obj p)))))
 
 #|
 (define (pretty-print-all)
@@ -685,7 +690,7 @@
 (define* (pp-sequence seq)
   (let ((iter (make-iterator seq))
 	(strs ())
-	(plen (let-ref *s7* 'print-length)))
+	(plen (*s7* 'print-length)))
     (do ((i 0 (+ i 1))
 	 (entry (iterate iter) (iterate iter)))
 	((or (= i plen)
